@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 08:34:23 1995
  * Project:     INN (innfeed)
  * File:        config.h
- * RCSId:       $Id: config.h,v 1.16 1997/01/13 00:33:25 brister Exp $
+ * RCSId:       $Id: config.h,v 1.17 1997/02/13 22:52:58 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -30,58 +30,78 @@
  * 
  */
 
-
-#if 0
-
-Many of the values in here will be eventually moved to a run-time config
-file.
-
-#endif
-
-
 #if ! defined ( config_h__ )
 #define config_h__
 
 /* Go edit sysconfig.h for platform differences */
 #include "sysconfig.h"
 
-/* set this to 1 if you want innfeed.status to be wrapped in HTML code. */
-#define GEN_HTML 0
-
 /**********************************************************************/
 /*                     Application specific defines                   */
 /**********************************************************************/
 
-/* same as INN's value. Yes, this will eventually come from inn's config
-   files. */
-#define NEWSSPOOL "/var/news/spool/articles"
-
-/* where the backlog files get stored. */
-#define TAPE_DIRECTORY "/var/news/spool/innfeed"
-
 /* the path to the run-time config file. If relative, then relative to
    TAPE_DIRECTORY (or the value of the ``-b'' option). Overridden by ``-c''
    option. */
-#define CONFIG_FILE "innfeed.conf"
+#define CONFIG_FILE "/var/news/etc/innfeed.conf"
 
-/* where to scribble the process id. If relative, then relative to
-   TAPE_DIRECTORY (or ``-b'' option value). Overridden by ``-p'' option */
-#define PID_FILE "innfeed.pid"
 
-/* where to write the status informatio. If relative then relative to
-   TAPE_DIRECTORY (or value of ``-b'' option). Overridden by ``-S''
-   option. */
-#define INNFEED_STATUS "innfeed.status"
+/*
+ * This next section contains things than can be overriden in the config
+ * file. The strings inside each comment is the key used in the
+ * innfeed.conf file to override the value here. See innfeed.conf for a
+ * description of each./
+ */
 
-/* the path to a log file where stdout and stderr will go if they're
-   pointing at /dev/null on program start. This will catch any fatal
-   messages that syslog didn't. Can override with '-l' option. If stderr
-   and stdout are going to a tty then this is not used. */
-#define LOG_FILE "/var/log/news/innfeed.log"
+/* in tape.c */
+#define TAPE_DIRECTORY 	      "/var/news/spool/innfeed" /* backlog-directory */
+#define TAPE_HIGHWATER 		5 		/* backlog-highwater */
+#define TAPE_ROTATE_PERIOD 	60 		/* backlog-rotate-period */
+#define TAPE_CHECKPOINT_PERIOD 	30 		/* backlog-ckpt-period */
+#define TAPE_NEWFILE_PERIOD 	600 		/* backlog-newfile-period */
 
-/* the full pathname of the file to get a printed dump of the system when
-   a SIGINT is delivered (or SNAPSHOT_ON_DIE is non-zero--see below). */
-#define SNAPSHOT_FILE "/var/tmp/innfeed.snapshot"
+/* in main.c */
+#define NEWSSPOOL 		"/var/news/spool/articles" /* news-spool */
+#define PID_FILE 		"innfeed.pid" 	/* pid-file */
+#define LOG_FILE 		"/var/log/news/innfeed.log" /* log-file */
+
+/* in host.c */
+#define DNS_RETRY_PERIOD 	900 		/* dns-retry */
+#define DNS_EXPIRE_PERIOD 	86400 		/* dns-expire */
+#define CLOSE_PERIOD 		(60 * 60) 	/* close-period */
+#define GEN_HTML		0 		/* gen-html */
+#define INNFEED_STATUS 		"innfeed.status" /* status-file */
+#define LOG_CONNECTION_STATS 	0 		/* connection-stats */
+#define HOST_HIGHWATER 		10 		/* host-highwater */
+#define STATS_PERIOD 		(60 * 10) 	/* stats-period */
+#define STATS_RESET_PERIOD 	(60 * 60 * 12) 	/* stats-reset-period */
+
+#define ARTTOUT		 	600 		/* article-timeout */
+#define RESPTOUT	 	300 		/* response-timeout */
+#define INIT_CXNS		1 		/* initial-connections */
+#define MAX_CXNS		2 		/* max-connections */
+#define MAX_Q_SIZE		5 		/* max-queue-size */
+#define STREAM			1 		/* streaming */
+#define NOCHECKHIGH 		95.0 		/* no-check-high */
+#define NOCHECKLOW 		90.0 		/* no-check-low */
+#define PORTNUM 		119 		/* port-number */
+#define BLOGLIMIT		0 		/* backlog-limit */
+#define LIMIT_FUDGE 		1.10 		/* backlog-factor */
+#define BLOGLIMIT_HIGH		0 		/* backlog-limit-high */
+
+#define INIT_RECON_PER 30 	/* initial-reconnect-time */
+#define MAX_RECON_PER (60 * 60 * 1)/* max-reconnect-time */
+
+
+
+
+
+
+
+/****************************************************************************/
+/* 
+ * The rest below are not run-time configurable.
+ */
 
 /* If this file exists at startup then it's the same as having done
    '-d 1' on the command line. This is a cheap way of avoiding continual
@@ -93,19 +113,16 @@ file.
    lot of disk space. */
 #define SNAPSHOT_ON_DIE 0
 
+/* the full pathname of the file to get a printed dump of the system when
+   a SIGINT is delivered (or SNAPSHOT_ON_DIE is non-zero--see below). */
+#define SNAPSHOT_FILE "/var/tmp/innfeed.snapshot"
+
 /* Define this be an existing directory (or NULL). If innfeed deliberatly
    dumps core it will chdir() to this directory first (if non-NULL). If
    NULL then it will chdir to TAPE_DIRECTORY (as possibly modified by
    the '-b' option). */
 #define CORE_DIRECTORY NULL
 
-/* Define this to be a real number > 1.0. If the size limit on output
-   backlog files is set, then when an output tape gets bigger than the
-   limit times this number, the output file will be shrunk back down to the
-   limit. If this is too close to 1.0, then shrinking will happen too
-   frequently. */
-#define LIMIT_FUDGE 1.10
-
 /* strings that get added to the end of a peer name for generating
    backlog file names.  A peername cannot end in any of these string
    (e.g. having a peer called 'mypeer.input' will not work) */
@@ -113,51 +130,16 @@ file.
 #define INPUT_TAIL ".input"
 #define LOCK_TAIL ".lock"
 
-/* number of articles a tape has in memory before it dumps them to disk */
-#define TAPE_HIGHWATER 5
-
-/* the default number of seconds between checkpointing the tapes. */
-#define TAPE_CHECKPOINT_PERIOD 30
-
-/* the maxmmum number of articles a host has backed up in its queue before
-   giving them to its tape. */
-#define HOST_HIGHWATER 10
-
 /* rough estimate of average article line length (including
    headers). Smaller number means more efficient article preparation (for
    transfer), but, if much smaller than reality, then more memory
    wastage. */
 #define CHARS_PER_LINE 60
 
-/* set this to 1 if you want all the connections to a host to log their
-   respective stats when the Host logs the summary */
-#define LOG_CONNECTION_STATS 0
-
-/* How many seconds between logging statistics on transfers */
-#define STATS_PERIOD (60 * 10)  /* 10 minutes */
-
-/* how many seconds before issuing a 'final' stats log and resetting all
-   the counters (if innfeed runs for more than a day it's tricky to
-   generate meaningful reports with innlog). */
-#define STATS_RESET_PERIOD (60 * 60 * 12) /* 12 hours */
-
 /* How many seconds between logging statistics on article allocation.
    For no logging set to 0 */
 #define ARTICLE_STATS_PERIOD (10 * 60) /* 10 minutes */
 
-/* The number of seconds we first wait for trying to reconnect on a
-   failure. */
-#define INITIAL_REESTABLISHMENT_PERIOD 30
-
-/* maximum number of seconds between restablishment attempt. The timeout
-   period doubles on each failure. */
-#define MAX_REESTABLISHMENT_PERIOD (60 * 60 * 1)/* 1 hour */
-
-/* Default number of seconds before closing down connections and setting
-   them back up again. This is because some nntp servers hold the history
-   file open and a long running innfeed gives those systems problems....  */
-#define CLOSE_PERIOD (60 * 60)   /* 1 hour */
-
 /* max number of parallel connections to a single remote. This is just a
    sanity check for the runtime config file. */
 #define MAX_CONNECTION_COUNT 50
@@ -178,13 +160,8 @@ file.
 #define MAX_MAXART_TOUT 86400   /* one day max between articles from inn */
 #define MAX_RESP_TOUT 3600      /* one hour max to wait for response */
 
-/* Range limits for low-pass filter for no-CHECK mode. Stay within
-   (0,+10) */
-#define LOW_PASS_FILTER_ON 8.0
-#define LOW_PASS_FILTER_OFF 7.5
-
 /* the maximum number of peers we'll handle (not connections) */
-#define MAX_HOSTS 50
+#define MAX_HOSTS 100
 
 /* If we have more that this much article contents in memory then we've
    probably got a memory leak so we print a snapshot and dump core. */
@@ -220,7 +197,7 @@ file.
 #define REMALLOC(ptr, SIZE) (char *) realloc((char *)(ptr),(SIZE))
 
 #undef FREE
-#define FREE(ptr) free((char *)(ptr))
+#define FREE(ptr) (free((char *)(ptr)), ptr = NULL)
 
 #else
 
@@ -251,7 +228,9 @@ file.
 #define __attribute__(x)
 #endif
   
+#if defined (CHECK_FORMATS)
 extern void syslog (int, const char *,...) __attribute__ ((__format__ (printf, 2, 3)));
+#endif
+
 
-  
 #endif /* config_h__ */
