@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 23:08:24 1995
  * Project:     INN (innfeed)
  * File:        endpoint.c
- * RCSId:       $Id: endpoint.c,v 1.21 1997/01/13 00:34:53 brister Exp $
+ * RCSId:       $Id: endpoint.c,v 1.22 1997/02/14 02:19:47 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -38,7 +38,7 @@
  */
 
 #if ! defined (lint)
-static char rcsid [] = "$Id: endpoint.c,v 1.21 1997/01/13 00:34:53 brister Exp $" ;
+static char rcsid [] = "$Id: endpoint.c,v 1.22 1997/02/14 02:19:47 brister Exp $" ;
 static void use_rcsid (const char *rId) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rId) ;
 } 
@@ -59,7 +59,10 @@ static void use_rcsid (const char *rId) {   /* Never called */
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+
+#if ! defined (DO_NEED_SYS_SELECT) 
 #include <sys/select.h>
+#endif
 
 #if defined (DO_NEED_TIME)
 #include <time.h>
@@ -104,6 +107,11 @@ struct timeval
 #endif
 
 
+#if ! defined (NSIG)
+#define NSIG 32
+#endif
+
+
   /* This is the structure that is the EndPoint */
 struct endpoint_s 
 {
@@ -152,23 +160,27 @@ typedef struct timerqelem_s
   /* set to 1 elsewhere if you want stderr to get what's also being written
      in doWrite. */
 int debugWrites ;
-void (*gPrintInfo) (void) ;
 
 extern const char *InputFile ;
 
-static bool sigFlag ;
+static int sigFlag ;
 static EndPoint mainEndPoint ;
 static bool mainEpIsReg = false ;
 
 time_t  PrivateTime;
 
 
+typedef void (*sigfn) (int) ;
+static sigfn *sigHandlers ;
+
+
+
   /* private functions */
 static IoStatus doRead (EndPoint endp) ;
 static IoStatus doWrite (EndPoint endp) ;
 static IoStatus doExcept (EndPoint endp) ;
 static void pipeHandler (int s) ;
-static void intHandler (int s) ;
+static void signalHandler (int s) ;
 static int hitCompare (const void *v1, const void *v2) ;
 static void reorderPriorityList (void) ;
 static TimerElem newTimerElem (TimeoutId i, time_t w, EndpTCB f, void *d) ;
@@ -195,6 +207,8 @@ static int keepSelecting ;
 
 static TimerElem timeoutQueue ;
 static TimeoutId nextId ;
+static int timeoutQueueLength ;
+
 
 
 
@@ -427,6 +441,33 @@ int prepareWrite (EndPoint endp,
 }
 
 
+/* cancel all pending writes. The first len bytes of the queued write are
+  copied to buffer. The number of bytes copied (if it is less than *len) is
+  copied to len. If no write was outstanding then len will have 0 stored in
+  it. */
+void cancelWrite (EndPoint endp, char *buffer, size_t *len)
+{
+  char *p ;
+  size_t l ;
+  
+  FD_CLR (endp->myFd, &wrSet) ;
+  if (!FD_ISSET (endp->myFd, &rdSet))
+    FD_CLR (endp->myFd, &wrSet) ;
+
+#if 0
+#error XXX need to copy data to buffer and *len
+#endif
+  
+  freeBufferArray (endp->outBuffer) ;
+  
+  endp->outBufferIdx = 0 ;
+  endp->outIndex = 0 ;
+  endp->outCbk = NULL ;
+  endp->outClientData = NULL ;
+  endp->outSize = 0 ;
+  endp->outAmtWritten = 0 ;
+}
+
 /* queue up a new timeout request. to go off at a specific time. */
 TimeoutId prepareWake (EndpTCB func, time_t timeToWake, void *clientData) 
 {
@@ -489,14 +530,16 @@ bool removeTimeout (TimeoutId tid)
     p->next = n->next ;
 
   FREE (n) ;
-        
+
+  timeoutQueueLength-- ;
+  
   return true ;
 }
 
 
 /* The main routine. This is a near-infinite loop that drives the whole
    program. */
-void Run () 
+void Run (void) 
 {
   fd_set rSet ;
   fd_set wSet ;
@@ -504,8 +547,7 @@ void Run ()
 
   keepSelecting = 1 ;
   signal (SIGPIPE, pipeHandler) ;
-  signal (SIGINT, intHandler) ;
-  
+
   while (keepSelecting)
     {
       struct timeval timeout ;
@@ -537,7 +579,7 @@ void Run ()
                 break ;
               }
         }
-      
+
       sval = select (highestFd + 1, &rSet, &wSet, &eSet, twait) ;
 
       timePasses () ;
@@ -546,11 +588,13 @@ void Run ()
         die ("No fd's ready and no timeouts") ;
       else if (sval < 0 && errno == EINTR)
         {
-          if (sigFlag == true && gPrintInfo != NULL)
-            {
-              gPrintInfo () ;
-              sigFlag = false ;
-            }
+          if (sigFlag > 0 && sigFlag < NSIG &&
+              sigHandlers[sigFlag] != NULL &&
+              sigHandlers[sigFlag] != SIG_IGN &&
+              sigHandlers[sigFlag] != SIG_DFL)
+            (sigHandlers[sigFlag])(sigFlag) ;
+
+          sigFlag = 0 ;
         }
       else if (sval < 0) 
         {
@@ -563,12 +607,17 @@ void Run ()
           int readyCount = sval ;
           int timesThrough = 0 ;
           
-          if (sigFlag == true && gPrintInfo != NULL)
-            {
-              gPrintInfo () ;
-              sigFlag = false ;
-            }
-          
+          if (sigFlag > 0 && sigFlag < NSIG &&
+              sigHandlers[sigFlag] != NULL &&
+              sigHandlers[sigFlag] != SIG_IGN &&
+              sigHandlers[sigFlag] != SIG_DFL)
+            (sigHandlers[sigFlag])(sigFlag) ;  
+          else if (sigFlag > 0 && sigFlag < NSIG)
+            syslog (LOG_WARNING,"ME Unhandled signal: %d\n",sigFlag) ;
+          else if (sigFlag > NSIG)
+            syslog (LOG_WARNING,"ME signal number too big: %d vs %d\n",
+                    sigFlag,NSIG) ;
+              
           for (idx = 0 ; idx < priorityCount ; idx++)
             {
               EndPoint ep = priorityList [idx] ;
@@ -755,7 +804,7 @@ void *addWorkCallback (EndPoint endp, EndpWorkCbk cbk, void *data)
 }
 
 /* Tell the Run routine to stop next time around. */
-void stopRun () 
+void stopRun (void) 
 {
   keepSelecting = 0 ;
 }
@@ -786,7 +835,7 @@ void setMainEndPoint (EndPoint endp)
   else if (endp->myFd < 0)
     syslog (LOG_ERR,"Negative fd for mainEndPoint???") ;
   else
-    mainEpIsReg = S_ISREG(buf.st_mode) ;
+    mainEpIsReg = (S_ISREG(buf.st_mode) ? true : false) ;
 }
 
 void freeTimeoutQueue (void)
@@ -799,6 +848,7 @@ void freeTimeoutQueue (void)
       n = p->next ;
       FREE (p) ;
       p = n ;
+      timeoutQueueLength-- ;
     }
 }
 
@@ -918,9 +968,19 @@ static IoStatus doRead (EndPoint endp)
           endp->myErrno = errno ;
           rval = IoFailed ;
         }
+      else if (i < 0 && errno == EINTR)
+        {
+          if (sigFlag > 0 && sigFlag < NSIG &&
+              sigHandlers[sigFlag] != NULL &&
+              sigHandlers[sigFlag] != SIG_IGN &&
+              sigHandlers[sigFlag] != SIG_DFL)
+            (sigHandlers[sigFlag])(sigFlag) ;
+
+          sigFlag = 0 ;
+        }
       else if (i == 0)
         rval = IoEOF ;
-      else                   /* i < 0 && errno == EINTR || errno == EAGAIN */
+      else                   /* i < 0 && errno == EAGAIN */
         rval = IoIncomplete ;
 
       FREE (vp) ;
@@ -1013,14 +1073,24 @@ static IoStatus doWrite (EndPoint endp)
           if (endp->outAmtWritten == endp->outSize)
             rval = IoDone ;
         }
-      else if (i < 0 && errno != EAGAIN)
+      else if (i < 0 && errno == EINTR)
         {
-          endp->myErrno = errno ;
-          rval = IoFailed ;
+          if (sigFlag > 0 && sigFlag < NSIG &&
+              sigHandlers[sigFlag] != NULL &&
+              sigHandlers[sigFlag] != SIG_IGN &&
+              sigHandlers[sigFlag] != SIG_DFL)
+            (sigHandlers[sigFlag])(sigFlag) ;
+          
+          sigFlag = 0 ;
+        }
+      else if (i < 0 && errno == EAGAIN)
+        {
+          rval = IoIncomplete ;
         }
       else if (i < 0)
         {
-          rval = IoIncomplete ;
+          endp->myErrno = errno ;
+          rval = IoFailed ;
         }
       else
         dprintf (1,"Wrote 0 bytes in doWrite()?\n") ;
@@ -1066,10 +1136,10 @@ static void endPointPrint (EndPoint ep, FILE *fp)
 }
 #endif
 
-static void intHandler (int s)
+static void signalHandler (int s)
 {
-  sigFlag = true ;
-  signal (s, intHandler) ;
+  sigFlag = s ;
+  signal (s, signalHandler) ;
 }
 
 
@@ -1173,6 +1243,8 @@ static TimeoutId timerElemAdd (time_t when, EndpTCB func, void *data)
       q->next = p ;
     }
 
+  timeoutQueueLength++ ;
+  
   return p->id ;
 }
 
@@ -1225,6 +1297,8 @@ static void doTimeout (void)
   timeoutQueue = timeoutQueue->next ;
 
   FREE (p) ;
+
+  timeoutQueueLength-- ;
   
   if (cbk)
     (*cbk) (tid, data) ;        /* call the callback function */
@@ -1475,3 +1549,27 @@ int main (int argc, char **argv)
   return 0;
 }
 #endif
+
+
+void setSigHandler (int signum, void (*ptr)(int))
+{
+  int i ;
+  
+  if (sigHandlers == NULL)
+    {
+      sigHandlers = ALLOC (sigfn,NSIG) ;
+      for (i = 0 ; i < NSIG ; i++)
+        sigHandlers [i] = NULL ;
+    }
+
+  if (signum >= NSIG)
+    {
+      syslog (LOG_ERR,"ME signal number bigger than NSIG: %d vs %d",
+              signum,NSIG) ;
+      return ;
+    }
+
+  (void) signal (signum,signalHandler) ;
+  sigHandlers[signum] = ptr ;
+}
+
