@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 22:04:49 1995
  * Project:     INN (innfeed)
  * File:        endpoint.h
- * RCSId:       $Id: endpoint.h,v 1.6 1997/01/07 21:02:21 brister Exp $
+ * RCSId:       $Id: endpoint.h,v 1.7 1997/02/14 02:20:18 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -96,8 +96,6 @@
  *
  */
 
-extern void (*gPrintInfo) (void) ;
-
 /* create a new EndPoint hooked up to the given file descriptor */
 EndPoint newEndPoint (int fd) ;
 
@@ -135,6 +133,9 @@ int prepareWrite (EndPoint endp,
                   EndpRWCB func,
                   void *clientData) ;
 
+/* cancel any outstanding writes. */
+void cancelWrite (EndPoint endp, char *buffer, size_t *len) ;
+
 /* return true if prepareRead has been called, but not serviced yet */
 bool readIsPending (EndPoint endp) ;
 
@@ -156,6 +157,8 @@ TimeoutId prepareSleep (EndpTCB func,
      been serviced. */
 void *addWorkCallback (EndPoint endp, EndpWorkCbk cbk, void *data) ;
 
+void setSigHandler (int sig, void (*)(int)) ;
+
 /* remove the timeout that was previously requested. Retuesn true if
    succesfully removed, false otherwise. 0 is a legal parameter value, in
    which case the function simply returns. */
