@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 08:25:26 1995
  * Project:     INN (innfeed)
  * File:        innlistener.h
- * RCSId:       $Id: innlistener.h,v 1.9 1997/01/13 00:39:45 brister Exp $
+ * RCSId:       $Id: innlistener.h,v 1.10 1997/02/13 23:14:37 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -44,31 +44,36 @@
 
 extern InnListener mainListener ;
 
-  /* Initialization of the InnListener object. If it fails then returns
-     NULL. ENDPOINT is the endpoint where the article info will come
-     from. A dummy listener exists when processing backlog files and is
-     there just to help drive the process. */
+/* Initialization of the InnListener object. If it fails then returns
+  NULL. ENDPOINT is the endpoint where the article info will come
+  from. A dummy listener exists when processing backlog files and is
+  there just to help drive the process. */
 InnListener newListener (EndPoint endp, bool isDummy, bool dynamicPeers) ;
 
-  /* print some useful debugging information about the Listener and all its
-     Hosts and all their Connecitons/Article/Buffers etc. to the given
-     FILE. */
+/* print some useful debugging information about the Listener and all its
+ * Hosts and all their Connecitons/Article/Buffers etc. to the given FILE.
+ */
 void gPrintListenerInfo (FILE *fp, u_int indentAmt) ;
 void printListenerInfo (InnListener listener, FILE *fp, u_int indentAmt) ;
 
-  /* Called by the Host when it is about to delete itself */
+/* Called by the Host when it is about to delete itself */
 u_int listenerHostGone (InnListener listener, Host host) ;
 
-    /* Called to hook up the given Host to the Listener. */
+/* Called to hook up the given Host to the Listener. */
 bool listenerAddPeer (InnListener listener, Host hostObj) ;
 
-  /* true if the listener is a dummy. */
+/* true if the listener is a dummy. */
 bool listenerIsDummy (InnListener listener) ;
 
-  /* This gets called to stop accepting new articles from innd. Typically
-     called by the Commander */
+/*
+ * This gets called to stop accepting new articles from innd. Typically
+ * called by the signal handler, or when the listener gets EOF on its input
+ * (in channel mode)
+ */
 void shutDown (InnListener cxn) ;
 
+/* Callback fired after config file is loaded */
+int listenerConfigLoadCbk (void *data) ;
 
   /* stop a specific host. */
 void shutDownHost (InnListener cxn, const char *peerName) ;
