@@ -4,7 +4,7 @@
  * Start Date:  Mon, 15 Jan 1996 17:31:58 +1100
  * Project:     INN -- innfeed
  * File:        main.c
- * RCSId:       $Id: main.c,v 1.23 1997/01/13 00:40:41 brister Exp $
+ * RCSId:       $Id: main.c,v 1.24 1997/02/13 23:21:31 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,13 +29,12 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: main.c,v 1.23 1997/01/13 00:40:41 brister Exp $" ;
+static const char *rcsid = "$Id: main.c,v 1.24 1997/02/13 23:21:31 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
 #endif
 
-
 #include "config.h"             /* system specific configuration */
 
 #include <stdlib.h>
@@ -68,26 +67,29 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include "msgs.h"
 #include "buffer.h"
 #include "connection.h"
-
+#include "configfile.h"
 
 #if defined(DO_HAVE_UNIX_DOMAIN)
 #include <sys/un.h>
 #endif  /* defined(DO_HAVE_UNIX_DOMAIN) */
 
-
 #include "endpoint.h"
 #include "host.h"
 #include "innlistener.h"
 
+#define INHERIT 1
+#define NO_INHERIT 0
 
 /* exports */
+bool talkToSelf ;
 extern int debugWrites ;
 bool sigFlag = false ;
 const char *InputFile ;
 const char *configFile = NULL ;
 bool RollInputFile = false ;
-const char *pidFile = PID_FILE ;
+const char *pidFile = NULL ;
 bool useMMap = false ;
+void (*gPrintInfo) (void) ;
 
 /* imports */
 extern char *versionInfo ;
@@ -99,12 +101,14 @@ extern int optind;
 extern void openInputFile (void);
 
 /* privates */
-static bool talkToSelf ;
 static char *logFile ;
-static const char *newsspool = NEWSSPOOL;
+static char *newsspool ;
 
+static void sigiot (int sig) ;
 static void sigalrm (int sig) ;
 static void sigchld (int sig) ;
+static void sigint (int sig) ;
+static void sigquit (int sig) ;
 static void sighup (int sig) ;
 static void sigterm (int sig) ;
 static void sigusr (int sig) ;
@@ -112,21 +116,34 @@ static void usage (int) ;
 static void gprintinfo (void) ;
 static void openLogFile (void) ;
 static void writePidFile (void) ;
+static int mainOptionsProcess (void *data) ;
+static int mainConfigLoadCbk (void *data) ;
+
+static char *bopt = NULL ;
+static char *aopt = NULL ;
+static char *popt = NULL ;
+static bool Mopt = false ;
+static bool Zopt = false ;
+static bool Dopt = false ;
+static int debugLevel = 0 ;
+static u_int initialSleep = 2 ;
+static char *sopt = NULL ;
+static char *lopt = NULL ;
+static bool eopt = false ;
+static int elimit = 0 ;
 
 int main (int argc, char **argv)
 {
   EndPoint ep ;
   InnListener listener ;
-  int optVal, fd ;
+  int optVal, fd, rval ;
   const char *subProgram = NULL ;
   bool seenV = false ;
   bool dynamicPeers = false ;
   time_t now = theTime() ;
   char dateString [30] ;
-  char *sopt = NULL ;
   char *copt = NULL ;
-  char *lopt = NULL ;
-  int lim ;
+  bool checkConfig = false ;
 
   strcpy (dateString,ctime(&now)) ;
   dateString [24] = '\0' ;
@@ -148,20 +165,24 @@ int main (int argc, char **argv)
      data being in NNTP format. */
   artBitFiddleContents (true) ;
 
-#define OPT_STRING "a:b:c:d:e:hl:mMo:p:S:s:vxyz"
+#define OPT_STRING "a:b:c:Cd:e:hl:mMo:p:S:s:vxyz"
 
   while ((optVal = getopt (argc,argv,OPT_STRING)) != EOF)
     {
       switch (optVal) 
         {
           case 'a':
-            newsspool = optarg ;
+            aopt = optarg ;
             break ;
 
           case 'b':
             if ( !isDirectory (optarg) )
               logAndExit (1,"Not a directory: %s\n",optarg) ;
-            setTapeDirectory (optarg) ;
+            bopt = optarg ;
+            break ;
+
+          case 'C':
+            checkConfig = true ;
             break ;
 
           case 'c':
@@ -170,18 +191,20 @@ int main (int argc, char **argv)
 
           case 'd':
             loggingLevel = atoi (optarg) ;
+            debugLevel = loggingLevel ;
+            Dopt = true ;
             break ;
 
           case 'e':
-            lim = atoi (optarg) ;
-            if (lim <= 0)
+            eopt = true ;
+            elimit = atoi (optarg) ;
+            if (elimit <= 0)
               {
                 fprintf (stderr,"Illegal value for -e option\n") ;
                 usage (1) ;
               }
-            setOutputSizeLimit (atoi (optarg)) ;
             break ;
-            
+
           case 'h':
             usage (0) ;
 
@@ -190,9 +213,10 @@ int main (int argc, char **argv)
             break ;
 
           case 'M':
+            Mopt = true ;
             useMMap = false ;
             break ;
-            
+
           case 'm':
             artLogMissingArticles (true) ;
             break ;
@@ -202,7 +226,7 @@ int main (int argc, char **argv)
             break ;
 
           case 'p':
-            pidFile = optarg ;
+            popt = optarg ;
             break ;
 
           case 's':
@@ -220,15 +244,15 @@ int main (int argc, char **argv)
           case 'x':
             talkToSelf = true ;
             break ;
-            
+
           case 'y':
             dynamicPeers = true ;
             break ;
 
           case 'z':
-            hostLogConnectionStats (true) ;
+            Zopt = true ;
             break ;
-            
+
           default:
             usage (1) ;	
         }
@@ -249,42 +273,44 @@ int main (int argc, char **argv)
     }
 
   /* make sure we have valid fds 0, 1 & 2 so it is not taken by
-     something else, probably openlog().  fd 0 will be freopen()ed on the
-     inputFile, the subProgram, or ourself.  fd 1 and fd 2 will
-     be freopen()ed on the log file (or will stay pointed at /dev/null).
-
-     without doing this, if the descriptors were closed then the
-     freopen calls on some systems (like BSDI 2.1) will really close
-     whatever has aquired the stdio descriptors, such as the socket
-     to syslogd.
-
-     XXX possible problems:  what if fd 0 is closed but no inputFile,
-     XXX subProgram or talkToSelf is true?  it will not be freopen()ed, so
-     XXX innfeed won't have any fresh data (besides, fd 0 is only writable
-     XXX here).  perhaps a warning should be issued.
-   */
+    something else, probably openlog().  fd 0 will be freopen()ed on the
+    inputFile, the subProgram, or ourself.  fd 1 and fd 2 will
+    be freopen()ed on the log file (or will stay pointed at /dev/null).
+    
+    without doing this, if the descriptors were closed then the
+    freopen calls on some systems (like BSDI 2.1) will really close
+    whatever has aquired the stdio descriptors, such as the socket
+    to syslogd.
+    
+    XXX possible problems:  what if fd 0 is closed but no inputFile,
+    XXX subProgram or talkToSelf is true?  it will not be freopen()ed, so
+    XXX innfeed won't have any fresh data (besides, fd 0 is only writable
+    XXX here).  perhaps a warning should be issued.
+    */
   do
     {
       fd = open("/dev/null", O_WRONLY);
       switch (fd)
         {
-        case -1:
-          logAndExit (1,"open(\"/dev/null\", O_WRONLY): %s", strerror (errno));
-          break;
-        case 0:
-        case 1:
-        case 2:
-          /* good, we saved an fd from being trounced */
-          break;
-        default:
-          close(fd);
+          case -1:
+            logAndExit (1,"open(\"/dev/null\", O_WRONLY): %s",
+                        strerror (errno));
+            break;
+          case 0:
+          case 1:
+          case 2:
+            /* good, we saved an fd from being trounced */
+            break;
+          default:
+            close(fd);
         }
     } while (fd < 2);
 
-  openlog (program,(int)(L_OPENLOG_FLAGS|LOG_PID),LOG_NEWS) ;
-
-  if (talkToSelf)
-    setOutputSizeLimit (0) ;
+  if ( !checkConfig ) 
+    {
+      openlog (program,(int)(L_OPENLOG_FLAGS|LOG_PID),LOG_NEWS) ;
+      syslog (LOG_NOTICE,STARTING_PROGRAM,versionInfo,dateString) ;
+    }
 
   if (subProgram == NULL && talkToSelf == false)
     {
@@ -293,9 +319,9 @@ int main (int argc, char **argv)
       if (fstat (0,&buf) < 0)
         logAndExit (1,FSTAT_FAILURE,"stdin") ;
       else if (S_ISREG (buf.st_mode))
-          InputFile = "";
+        InputFile = "";
     }
-  
+
   if (subProgram != NULL && (talkToSelf == true || InputFile))
     {
       dprintf (0,"Cannot specify '-s' with '-x' or an input file\n") ;
@@ -303,43 +329,56 @@ int main (int argc, char **argv)
       usage (1) ;
     }
 
-  if (sopt != NULL)
-    hostSetStatusFile (sopt) ;
+  /*
+   * set up the config file name and then read the file in. Order is important.
+   */
+  configAddLoadCallback (mainOptionsProcess,(checkConfig ? stderr : NULL)) ;
+  configAddLoadCallback (tapeConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
+
+  configAddLoadCallback (hostConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
+  configAddLoadCallback (cxnConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
+  configAddLoadCallback (mainConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
+  configAddLoadCallback (listenerConfigLoadCbk,(checkConfig ? stderr : NULL));
 
   if (copt != NULL && *copt == '\0')
-    logAndExit (1,"Empty pathname for ``-c'' option") ;
+    {
+      logOrPrint (LOG_CRIT,(checkConfig ? stderr : NULL),
+                  "Empty pathname for ``-c'' option") ;
+      exit (1) ;
+    }
+  else if (copt != NULL && bopt != NULL)
+    configFile = buildFilename (bopt,copt) ;
+  else if (bopt != NULL)
+    configFile = buildFilename (bopt,CONFIG_FILE) ;
   else if (copt != NULL)
-    configFile = buildFilename (getTapeDirectory(),copt) ;
+    configFile = buildFilename (TAPE_DIRECTORY,copt) ;
   else
-    configFile = buildFilename (getTapeDirectory(),CONFIG_FILE) ;
-  
-  if (lopt != NULL && *lopt == '\0')
-    logAndExit (1,"empty pathanme for ``-l'' option") ;
-  else if (lopt != NULL)
-    logFile = buildFilename (getTapeDirectory(),lopt) ;
-  
+    configFile = buildFilename (TAPE_DIRECTORY,CONFIG_FILE) ;
+
+  rval = readConfig (configFile,(checkConfig ? stderr : NULL),
+                     checkConfig,loggingLevel > 0);
+  if (checkConfig)
+    {
+      if (!rval)
+        fprintf (stderr,"config loading failed.\n") ;
+      else
+        fprintf (stderr,"config loading succeeded.\n") ;
+      exit (1) ;
+    }
+  else if (!rval)
+    exit (1) ;
+
   if (loggingLevel == 0 && fileExistsP (DEBUG_FILE))
     loggingLevel = 1 ;
 
-  if (logFile == NULL && ! isatty ( fileno (stderr) ) )
-    logFile = LOG_FILE ;
+  if (logFile == NULL && ! isatty (fileno (stderr)))
+    logFile = strdup (LOG_FILE) ;
 
   if (logFile)
     openLogFile () ;
 
-  syslog (LOG_NOTICE,STARTING_PROGRAM,versionInfo,dateString) ;
-
   openfds = 4 ;                 /* stdin, stdout, stderr and syslog */
-  
-  if ( !isDirectory (getTapeDirectory()) )
-    {
-      syslog (LOG_ERR,NOT_A_DIR,getTapeDirectory()) ;
-      dprintf (1,"Not a directory : %s\n",getTapeDirectory()) ;
-      exit (1) ;
-    }
-  
-  /* XXX this pid file should be done better to allow for multiple
-     XXX concurrent innfeed processes */
+
   writePidFile ();
 
   if (subProgram != NULL)
@@ -405,7 +444,6 @@ int main (int argc, char **argv)
       exit (1) ;
     }
 
-
     /* hook up the endpoint to the source of new article information (usually
        innd). */
   ep = newEndPoint (0) ;        /* fd 0, i.e. stdin */
@@ -414,10 +452,10 @@ int main (int argc, char **argv)
        possible activity. */
   setMainEndPoint (ep) ;
 
-  tapeSetCheckpointPeriod (TAPE_CHECKPOINT_PERIOD) ;
-
   listener = newListener (ep, talkToSelf,dynamicPeers) ;
   mainListener = listener ;
+
+  sleep (initialSleep) ;
   
   configHosts (talkToSelf) ;
 
@@ -426,24 +464,30 @@ int main (int argc, char **argv)
   }
 
   /* handle signal to shutndown */
-  signal (SIGTERM,sigterm) ;
+  setSigHandler (SIGTERM,sigterm) ;
+  setSigHandler (SIGQUIT,sigquit) ;
 
   /* handle signal to reload config */
-  signal (SIGHUP,sighup) ;
+  setSigHandler (SIGHUP,sighup) ;
+
+  /* handle signal to print snapshot. */
+  setSigHandler (SIGINT,sigint) ;
 
   /* handle signal to roll input file */
-  signal (SIGALRM,sigalrm) ;
+  setSigHandler (SIGALRM,sigalrm) ;
+
+  /* handle signal to flush all the backlog files */
+  setSigHandler (SIGIOT,sigiot) ;
 
   /* we can increment and decrement logging levels by sending SIGUSR{1,2} */
-  signal (SIGUSR1,sigusr) ;
-  signal (SIGUSR2,sigusr) ;
+  setSigHandler (SIGUSR1,sigusr) ;
+  setSigHandler (SIGUSR2,sigusr) ;
 
   Run () ;
-  
+
   exit (0) ;
 }
 
-
 static void usage (int val)
 {
   fprintf (stderr,"usage: %s [ options ] [ file ]\n\n",
@@ -460,13 +504,14 @@ static void usage (int val)
   fprintf (stderr,"\t-c file     Use the given file as the config file instead of the\n");
   fprintf (stderr,"\t            default of %s\n",CONFIG_FILE);
 
+  fprintf (stderr,"\t-C          Check the config file and then exit.\n") ;
   fprintf (stderr,"\t-d num      set the logging level to num (an integer).\n");
   fprintf (stderr,"\t            Larger value means more logging. 0 means no\n");
   fprintf (stderr,"\t            logging. The default is 0\n");
 
   fprintf (stderr,"\t-e bytes    Keep the output backlog files to no bigger\n");
   fprintf (stderr,"\t            than %.2f times this number\n",LIMIT_FUDGE);
-  
+
   fprintf (stderr,"\t-h          print this message\n");
 
   fprintf (stderr,"\t-l file     redirect stderr and stdout to the given file.\n");
@@ -510,34 +555,66 @@ static void usage (int val)
 
 static void sigterm (int sig)
 {
+  (void) sig ;
+  
   syslog(LOG_NOTICE, SHUTDOWN_SIGNAL);
   shutDown (mainListener) ;
+}
 
-  signal (sig,sigterm) ;
+static void sigquit (int sig)
+{
+  (void) sig ;
+  
+  sigterm (0) ;
+}
+
+static void sigint (int sig)
+{
+  (void) sig ;
+
+  gprintinfo () ;
 }
 
 static void sighup (int sig)
 {
-  /* XXX Just set variable and roll this into Run () ? */
+  (void) sig ;
+  
   syslog(LOG_NOTICE, CONFIG_RELOAD, configFile);
-  configHosts (talkToSelf) ;
 
-  signal (sig,sighup) ;
+  if (!readConfig (configFile,NULL,false,loggingLevel > 0))
+    {
+      syslog (LOG_ERR,PARSE_FAILURE) ;
+      exit (1) ;
+    }
+
+  configHosts (talkToSelf) ;
+}
+
+static void sigiot (int sig)
+{
+  (void) sig ;
+
+  gFlushTapes () ;
 }
 
 static void sigalrm (int sig)
 {
-  RollInputFile = true;
-  syslog(LOG_NOTICE, "ME preparing to roll %s", InputFile);
-
-  signal (sig,sigalrm) ;
+  (void) sig ;
+  
+  if (InputFile == NULL)
+    syslog (LOG_ERR,IGNORE_SIGALRM) ;
+  else 
+    {
+      RollInputFile = true;
+      syslog(LOG_NOTICE, "ME preparing to roll %s", InputFile);
+    }
 }
 
 static void sigchld (int sig)
 {
 
   (void) sig ;                  /* keep lint happy */
-  
+
 #if 0
   wait (&status) ;              /* we don't care */
 #endif
@@ -545,7 +622,6 @@ static void sigchld (int sig)
   signal (sig,sigchld) ;
 }
 
-
   /* SIGUSR1 increments logging level. SIGUSR2 decrements. */
 static void sigusr (int sig)
 {
@@ -556,11 +632,9 @@ static void sigusr (int sig)
     syslog(LOG_NOTICE, DECR_LOGLEVEL, loggingLevel);
     loggingLevel-- ;
   }    
-
-  signal (sig,sigusr) ;
 }
 
-static void openLogFile ()
+static void openLogFile (void)
 {
   FILE *fpr ;
 
@@ -570,12 +644,12 @@ static void openLogFile ()
       if (fpr != stdout)
         logAndExit (1,"freopen (%s, \"a\", stdout): %s",
                     logFile, strerror (errno)) ;
-
+      
       fpr = freopen (logFile,"a",stderr) ;
       if (fpr != stderr)
         logAndExit (1,"freopen (%s, \"a\", stderr): %s",
                     logFile, strerror (errno)) ;
-
+      
 #if defined (DO_HAVE_SETBUFFER)
       setbuffer (stdout, NULL, 0) ;
       setbuffer (stderr, NULL, 0) ;
@@ -586,14 +660,13 @@ static void openLogFile ()
     }
 }
 
-static void writePidFile ()
+static void writePidFile (void)
 {
   FILE *F;
   int pid;
-  char path [PATH_MAX] ;
 
   pidFile = buildFilename (getTapeDirectory(),pidFile) ;
-  
+
   if (pidFile == NULL)
     logAndExit (1,"NULL pidFile\n") ;
 
@@ -637,7 +710,6 @@ static void gprintinfo (void)
   setbuf (fp, NULL) ;
 #endif
 
-
   fprintf (fp,"----------------------------System snaphot taken at: %s\n",
            ctime (&now)) ;
   gPrintListenerInfo (fp,0) ;
@@ -653,3 +725,161 @@ static void gprintinfo (void)
   fclose (fp) ;
 }
 
+/* called after the config file is loaded and after the config data has
+  been updated with command line options. */
+static int mainConfigLoadCbk (void *data)
+{
+  FILE *fp = (FILE *) data ;
+  char *p ;
+  long ival ;
+  int bval ;
+
+  if (getString (topScope,"news-spool", &p,NO_INHERIT))
+    {
+      if ( !isDirectory (p) && isDirectory (NEWSSPOOL) )
+        {
+          logOrPrint (LOG_WARNING,fp,BADSPOOL_CHANGE,p,NEWSSPOOL) ;
+          p = strdup (NEWSSPOOL) ;
+        }
+      else if (!isDirectory (p))
+        logAndExit (1,"Bad spool directories: %s, %s\n",p,NEWSSPOOL) ;
+    }
+  else if (!isDirectory (NEWSSPOOL))
+    logAndExit (1,SPOOL_NODEF,NEWSSPOOL);
+  else
+    p = strdup (NEWSSPOOL) ;
+  newsspool = p ;
+
+  /***************************************************/
+  
+  if (getString (topScope,"pid-file",&p,NO_INHERIT))
+    pidFile = buildFilename (getTapeDirectory(),p) ;
+  else
+    pidFile = PID_FILE ;
+
+  
+  if (getInteger (topScope,"debug-level",&ival,NO_INHERIT))
+    loggingLevel = (u_int) ival ;
+  
+  
+  if (getInteger (topScope,"initial-sleep",&ival,NO_INHERIT))
+    initialSleep = (u_int) ival ;
+  
+  
+  if (getBool (topScope,"use-mmap",&bval,NO_INHERIT))
+    useMMap = (bval ? true : false) ;
+
+  
+  if (getString (topScope,"log-file",&p,NO_INHERIT))
+    logFile = buildFilename (getTapeDirectory(),p) ;
+
+  return 1 ;
+}
+
+/*
+ * called after config file is loaded but before other callbacks, so we
+ * can adjust config file values from options. They will be validated in the
+ * second callback.
+ */
+static int mainOptionsProcess (void *data)
+{
+  value *v ;
+
+  (void) data ;
+  
+  if (bopt != NULL)
+    {
+      if ((v = findValue (topScope,"backlog-directory",NO_INHERIT)) != NULL) 
+        {
+          FREE (v->v.charp_val) ;
+          v->v.charp_val = strdup (bopt) ;
+        }
+      else
+        addString (topScope,"backlog-directory",strdup (bopt)) ;
+    }
+
+  if (aopt != NULL)
+    {
+      if ((v = findValue (topScope,"news-spool",NO_INHERIT)) != NULL)
+        {
+          FREE (v->v.charp_val) ;
+          v->v.charp_val = strdup (aopt) ;
+        }
+      else
+        addString (topScope,"news-spool",strdup (aopt)) ;
+    }
+
+  if (sopt != NULL)
+    {
+      if ((v = findValue (topScope,"status-file",NO_INHERIT)) != NULL)
+        {
+          FREE (v->v.charp_val) ;
+          v->v.charp_val = strdup (sopt) ;
+        }
+      else
+        addString (topScope,"status-file",strdup (sopt)) ;
+    }
+
+
+  if (Dopt)
+    {
+      if ((v = findValue (topScope,"debug-level",NO_INHERIT)) != NULL)
+        v->v.int_val = debugLevel ;
+      else
+        addInteger (topScope,"debug-level",debugLevel) ;
+    }
+
+  
+  if (eopt || talkToSelf)
+    {
+      if (talkToSelf)
+        elimit = 0 ;
+      
+      if ((v = findValue (topScope,"backlog-limit",NO_INHERIT)) != NULL)
+        v->v.int_val = elimit ;
+      else
+        addInteger (topScope,"backlog-limit",debugLevel) ;
+    }
+
+  
+  if (Mopt)
+    {
+      if ((v = findValue (topScope,"use-mmap",NO_INHERIT)) != NULL)
+        v->v.bool_val = 0 ;
+      else
+        addBoolean (topScope,"use-mmap",0) ;
+    }
+  
+
+  if (popt != NULL)
+    {
+      if ((v = findValue (topScope,"pid-file",NO_INHERIT)) != NULL)
+        {
+          FREE (v->v.charp_val) ;
+          v->v.charp_val = strdup (popt) ;
+        }
+      else
+        addString (topScope,"pid-file",strdup (popt)) ;
+    }
+
+  if (Zopt)
+    {
+      if ((v = findValue (topScope,"connection-stats",NO_INHERIT)) != NULL)
+        v->v.bool_val = 1 ;
+      else
+        addBoolean (topScope,"connection-stats",1) ;
+    }
+
+  if (lopt != NULL)
+    {
+      if ((v = findValue (topScope,"log-file",NO_INHERIT)) != NULL)
+        {
+          FREE (v->v.charp_val) ;
+          v->v.charp_val = strdup (lopt) ;
+        }
+      else
+        addString (topScope,"log-file",strdup (lopt)) ;
+    }
+
+  return 1 ;
+}
