@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 17:10:18 1995
  * Project:     INN (innfeed)
  * File:        misc.c
- * RCSId:       $Id: misc.c,v 1.14 1997/01/13 00:41:31 brister Exp $
+ * RCSId:       $Id: misc.c,v 1.15 1997/02/14 02:26:55 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: misc.c,v 1.14 1997/01/13 00:41:31 brister Exp $" ;
+static const char *rcsid = "$Id: misc.c,v 1.15 1997/02/14 02:26:55 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -55,6 +55,7 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include <errno.h>
 #include <syslog.h>
 #include <sys/param.h>
+#include <limits.h>
 
 #if defined (DO_HAVE_UNISTD)
 #include <unistd.h>
@@ -79,6 +80,7 @@ char *program ;                 /* this should be set to argv[0] */
 int debuggingOutput ;
 u_int loggingLevel ;
 char **PointersFreedOnExit ;
+static void log (int level, const char *fmt, va_list args) ;
 
 
 
@@ -171,7 +173,7 @@ bool debuggingDump = true ;
 extern void (*gPrintInfo) (void) ;
 void (*gCleanUp) (void) = 0 ;
 
-void log (int level, const char *fmt, char *args)
+static void log (int level, const char *fmt, va_list args)
 {
   time_t now = time (NULL) ;
   char timeString [30] ;
@@ -191,11 +193,29 @@ void log (int level, const char *fmt, char *args)
   syslog (level,p) ;
 }
 
+void logOrPrint (int level, FILE *fp, const char *fmt, ...)
+{
+  va_list ap ;
+
+  va_start (ap,fmt) ;
+  if (fp != NULL)
+    {
+      vfprintf (fp,fmt,ap) ;
+      fputc ('\n',fp) ;
+    }
+  else
+    {
+      char buffer [512] ;      /* gag me */
+
+      vsprintf (buffer,fmt,ap) ;
+      syslog (level,buffer) ;
+    }
+  va_end (ap) ;
+}
+
 void die (const char *fmt, ...)
 {
   va_list ap ;
-  int out = 0 ;
-  char *p ;
 
   va_start (ap, fmt) ;
   log (LOG_ERR,fmt,ap) ;
@@ -805,7 +825,7 @@ char *buildFilename (const char *directory, const char *fname)
   
   len = strlen (directory) + strlen (fname) + 2 ;
 
-  if (len < PATH_MAX - 2)
+  if (len < pathMax(directory) - 2)
     {
       p = malloc (len + 1) ;
       p [0] = '\0' ;
@@ -824,21 +844,23 @@ char *buildFilename (const char *directory, const char *fname)
 
 
 /* borrows heavily from the shrinkfile program by chongo. */
-bool shrinkfile (FILE *fp, long size, char *name, char *mode)
+bool shrinkfile (FILE *fp, long size, char *name, const char *mode)
 {
   long currlen = ftell (fp) ;
-  char tmpname [PATH_MAX] ;
+  char *tmpname ;
   char buffer [BUFSIZ] ;
   FILE *tmpFp ;
-  FILE *ifile ;
   int c ;
   int i ;
 
+  tmpname = malloc (pathMax(NULL) + 1) ;
   sprintf (tmpname,"%s.XXXXXX",name) ;
   mktemp (tmpname) ;
 
   if (currlen <= size)
     {
+      FREE(tmpname) ;
+      
       dprintf (1,"No need to shrink file (%s %ld vs %ld\n",
                name,size,currlen) ;
       return true ;
@@ -848,6 +870,7 @@ bool shrinkfile (FILE *fp, long size, char *name, char *mode)
   if ((tmpFp = fopen (tmpname,"w")) == NULL)
     {
       syslog (LOG_ERR,SHRINK_TEMP_OPEN,tmpname) ;
+      FREE (tmpname) ;
       return false ;
     }
 
@@ -855,6 +878,7 @@ bool shrinkfile (FILE *fp, long size, char *name, char *mode)
     {
       fclose (tmpFp) ;
       syslog (LOG_ERR,SHRINK_SEEK,currlen - size,name) ;
+      FREE(tmpname) ;
       return false ;
     }
 
@@ -865,17 +889,19 @@ bool shrinkfile (FILE *fp, long size, char *name, char *mode)
         syslog (LOG_WARNING,SHRINK_NONL,name) ;
         fclose (tmpFp) ;
         fseek (fp,currlen,SEEK_SET) ;
+        FREE(tmpname) ;
         return false ;
     }
 
   /* copy the tail of the shrinking file to the temp file. */
   while ((i = fread (buffer,1,sizeof (buffer),fp)) > 0)
     {
-      if (fwrite (buffer,1,i,tmpFp) != i)
+      if (fwrite (buffer,1,i,tmpFp) != (size_t) i)
         {
           fclose (tmpFp) ;
           syslog (LOG_ERR,SHRINK_WRITETMP,tmpname) ;
           fseek (fp,currlen, SEEK_SET) ;
+          FREE(tmpname) ;
           return false ;
         }
     }
@@ -883,25 +909,64 @@ bool shrinkfile (FILE *fp, long size, char *name, char *mode)
   if (i < 0)
     logAndExit (1,SHRINK_READ,name) ;
 
+  fclose (tmpFp) ;
+
   if (unlink (name) != 0)
-    {
-      fclose (tmpFp) ;
-      logAndExit (1,UNLINK_FAILED,name) ;
-    }
+    logAndExit (1,UNLINK_FAILED,name) ;
 
   /* we're in the same directory so this is ok. */
   if (rename (tmpname,name) != 0)
     logAndExit (1,RENAME_FAILED,tmpname,name) ;
   
-  if ((ifile = freopen (name,mode,fp)) != fp)
+  if (freopen (name,mode,fp) != fp)
     logAndExit (1,SHRINK_FREOPEN,name) ;
 
   fseek (fp,0,SEEK_END) ;
   size = ftell (fp) ;
   
-  fclose (tmpFp) ;
-
   syslog (LOG_WARNING,FILE_SHRUNK,name,currlen,size) ;
 
+  FREE(tmpname) ;
+  
   return true ;
 }
+
+
+
+long pathMax (const char *pathname)
+{
+  static long rval = 0 ;
+
+  (void) pathname ;
+  
+  if (rval > 0)
+    return rval ;
+  
+#if defined (PATH_MAX)
+
+  rval = PATH_MAX ;
+
+#elif defined (_POSIX_PATH_MAX)
+
+  rval = _POSIX_PATH_MAX ;
+
+#elif defined (DO_HAVE_PATHCONF) && defined (_PC_PATH_MAX)
+
+  if (pathname == NULL)
+    pathname = "/tmp" ;
+  
+  rval = pathconf (pathname,_PC_PATH_MAX) ;
+
+#else
+
+  rval = 255 ;
+  if (!logged) 
+    {
+      syslog (LOG_ERR,NO_PATH_MAX,rval) ;
+      logged = true ;
+    }
+  
+#endif
+
+  return rval ;
+}
