@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 23:44:47 1995
  * Project:     INN (innfeed)
  * File:        misc.h
- * RCSId:       $Id: misc.h,v 1.13 1997/01/13 00:42:17 brister Exp $
+ * RCSId:       $Id: misc.h,v 1.14 1997/02/14 02:27:33 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -100,6 +100,9 @@ char *getMsgId (const char *p) ;
    the first blank (or NULL) after the string. Returns a pointer into PTR */
 char *findNonBlankString (char *ptr, char **tail) ;
 
+/* if fp is not NULL then print to it, otherwise syslog at the level. */
+void logOrPrint (int level, FILE *fp, const char *fmt, ...)
+  __attribute__ ((__format__ (printf, 3, 4)));
 
 /* print the arguments like printf to stderr then abort() */
 void die (const char *fmt, ...) __attribute__ ((__format__ (printf, 1, 2)));
@@ -167,7 +170,10 @@ char *buildFilename (const char *directory, const char *fname) ;
 /* string the file opened by FP to the give SIZE. The NEWNAME is the name
    of the file to have after truncation. FP will be reopened for writing on
    the new file and will be positioned at the end */
-bool shrinkfile (FILE *fp, long size, char *name, char *mode) ;
+bool shrinkfile (FILE *fp, long size, char *name, const char *mode) ;
+
+/* max length of pathname */
+long pathMax (const char *pathname) ;
 
 #define ASSERT(x) do{if (!(x))die("%s in file %s line %d",#x,__FILE__,__LINE__);}while(0)
 
