@@ -1,12 +1,17 @@
 /*
- * $Id: startinnfeed.c,v 1.5 1997/01/13 00:43:26 brister Exp $
+ * $Id: startinnfeed.c,v 1.7 1997/02/14 13:08:00 brister Exp $
  *
  * Start innfeed and pass it all the arguments given. Sets up process
  * limits for innfeed.
  */
 
+#if ! defined (USER)
 #define USER "news"
-#define INNFEED "/news/bin/innfeed"
+#endif
+
+#if ! defined (INNFEED)
+#define INNFEED "/usr/news/local/innfeed"
+#endif
 
 #include <pwd.h>                /* getpwent */
 #include <stdio.h>              /* fprintf */
@@ -19,14 +24,10 @@
 #include <sys/resource.h>       /* setrlimit */
 #include <string.h>
 
-static char *innfeed = INNFEED;
+static const char *innfeed = INNFEED;
 
 void
-main(ac, av, ep)
-     int ac;
-     char **av;
-     char **ep;
-
+main(int ac, char **av, char **ep)
 {
   struct passwd *pwd;
   struct rlimit rl;
@@ -59,7 +60,7 @@ main(ac, av, ep)
   else 
     {
       progname = av[0];
-      av[0] = innfeed;
+      av[0] = (char *) innfeed;
 
       if (execve(innfeed, av, ep) == -1)
         (void)fprintf(stderr, "%s: execve: %s\n",
