@@ -4,7 +4,7 @@
  * Start Date:  Thu, 01 Feb 1996 22:17:51 +1100
  * Project:     INN -- innfeed
  * File:        sysconfig.h
- * RCSId:       $Id: sysconfig.h,v 1.10 1997/01/07 14:08:44 brister Exp $
+ * RCSId:       $Id: sysconfig.h,v 1.11 1997/02/13 22:58:38 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -109,7 +109,10 @@
 #define DO_NEED_STREAM 1
 #define HAVE_MMAP
 
-#define wait3(a,b,c) waitpid(-1,a,b)
+ /* this may be needed for solaris version < 2.6 */
+/* #define wait3(a,b,c) waitpid(-1,a,b) */
+
+
 #endif /* defined (SOLARIS) */
 
 
@@ -131,6 +134,7 @@
 
 #if defined (__sgi)
 #define MAX_WRITEV_VEC 16    /* actually bigger on 5.2, but this on 5.3 and 6.x */
+#define HAVE_MMAP
 #endif
 
 
@@ -142,6 +146,7 @@
 #define DO_NEED_STRDUP 1
 #define MAX_WRITEV_VEC 16
 #define DO_NEED_STREAM 1
+#define NO_SBRK 1
 #endif
 
 
@@ -174,6 +179,13 @@
   /* Defaults below here. If you need to change something it should really
      be done in the architecture specific section just above. */
 
+/* define DONT_NEED_SYS_SELECT or DO_NEED_SYS_SELECT depending on whether
+   endpoint.c fails to compile properly and you have
+   /usr/include/sys/select.h */
+#if ! defined (DO_NEED_SYS_SELECT) && !defined (DONT_NEED_SYS_SELECT)
+#define DONT_NEED_SYS_SELECT 1
+#endif
+
 /* define DONT_NEED_STRDUP or DO_NEED_STRDUP depending on
    if you a strdup on your machine or not. */
 #if ! defined (DO_NEED_STRDUP) && ! defined (DONT_NEED_STRDUP)
@@ -272,8 +284,11 @@
 #endif
 
 
-/* how syslog should be opened. LOG_PID not needed--is added automatically. */
-#define L_OPENLOG_FLAGS (LOG_NDELAY | LOG_PERROR)
+/*
+ * how syslog should be opened. LOG_PID not needed--it is added
+ * automatically.
+ */
+#define L_OPENLOG_FLAGS (LOG_NDELAY)
 
 /* same as INN's define DO_HAVE_UNISTD or DONT_HAVE_UNISTD */
 #if ! defined (DO_HAVE_UNISTD) && ! defined (DONT_HAVE_UNISTD)
