@@ -4,7 +4,7 @@
  * Start Date:  Sun Dec 31 18:52:04 1995
  * Project:     INN (innfeed)
  * File:        tape.c
- * RCSId:       $Id: tape.c,v 1.19 1997/01/13 00:54:29 brister Exp $
+ * RCSId:       $Id: tape.c,v 1.20 1997/02/13 23:13:10 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -40,7 +40,7 @@
 
  
 #if ! defined (lint)
-static const char *rcsid = "$Id: tape.c,v 1.19 1997/01/13 00:54:29 brister Exp $" ;
+static const char *rcsid = "$Id: tape.c,v 1.20 1997/02/13 23:13:10 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -77,42 +77,21 @@ typedef struct direct DIRENTRY ;
 #endif
 #include <sys/time.h>
 
-
 #include "tape.h"
 #include "article.h"
 #include "endpoint.h"
 #include "msgs.h"
+#include "configfile.h"
 
 
-
-/* pathname of directory we store tape files in. */
-static char *tapeDirectory ;
-
-/* the callback ID of the checkpoint timer callback. */
-static TimeoutId checkPtId ;
-
-/* number of seconds between tape checkpoints. */
-static u_int tapeCkPtPeriod ;
-
-/* global list of tapes so we can checkpoint them periodically */
-static Tape *activeTapes ;
-
-/* Size of the activeTapes array */
-static size_t activeTapeSize ;
-
-/* index of last element in activeTapes that's being used. */
-static size_t activeTapeIdx ;
-
-/* default limit of the size of output tapes. */
-static long defaultSizeLimit ;
-
+#if 0
 /* a structure for temporary storage of articles. */
 typedef struct q_e_s
 {
     Article article ;
     struct q_e_s *next ;
 } *QueueElem ;
-
+#endif
 
 /* The Tape class type. */
 struct tape_s
@@ -135,20 +114,26 @@ struct tape_s
     FILE *inFp ;                /* input FILE */
     FILE *outFp ;               /* output FILE */
 
+    time_t lastRotated ;        /* time files last got switched */
+    bool checkNew ;             /* set bool when we need to check for
+                                   hand-crafted file. */
+    
+#if 0
     /* the tape holds a small output queue in memory to avoid thrashing. */
     QueueElem head ;            
     QueueElem tail ;
     u_int qLength ;             /* amount on the queue */
-
-    bool outputHasStuff ;       /* true if there are articles to be had */
-    bool outputFileExists ;     /* true if we've created the output file */
-
+#endif
+    
     long outputSize ;           /* the current size of the output file. */
+    long lossage ;
 
     /* the number of bytes we try to keep the output under. We actually
        wait for the outputSize to get 10% greater than this amount before
        shrinking the file down again. A value of zero means no limit. */
-    long outputLimit ;
+    long outputLowLimit ;
+    long outputHighLimit ;
+    double backlogFactor ;
     
     bool scribbled ;            /* have we scribbled a checkpoint value in
                                    the file. */
@@ -165,12 +150,159 @@ static void checkpointTape (Tape tape) ;
 static void removeTapeGlobally (Tape tape) ;
 static void addTapeGlobally (Tape tape) ;
 static void prepareFiles (Tape tape) ;
+static void tapeCkNewFileCbk (TimeoutId id, void *d) ;
 static void tapeCheckpointCallback (TimeoutId id, void *d) ;
+#if 0
+static void flushTape (Tape tape) ;
+#endif
+static void tapesSetCheckNew (void) ;
+static void initTape (Tape nt) ;
 
 
 
+/* pathname of directory we store tape files in. */
+static char *tapeDirectory ;
+
+/* the callback ID of the checkpoint timer callback. */
+static TimeoutId checkPtId ;
+static TimeoutId ckNewFileId ;
+
+/* number of seconds between tape checkpoints. */
+static u_int tapeCkPtPeriod ;
+static u_int tapeCkNewFilePeriod ;
+
+/* the callback ID of the tape rotation timer callback. */
+static TimeoutId rotateId ;
+
+static time_t rotatePeriod = TAPE_ROTATE_PERIOD ;
+
+/* global list of tapes so we can checkpoint them periodically */
+static Tape *activeTapes ;
+
+/* Size of the activeTapes array */
+static size_t activeTapeSize ;
+
+/* index of last element in activeTapes that's being used. */
+static size_t activeTapeIdx ;
+
+#if 0
+/* default limit of the size of output tapes. */
+static long defaultSizeLimit ;
+#endif
+
+u_int tapeHighwater ;
+
+bool debugShrinking = false ;
+
+extern bool talkToSelf ;        /* main.c */
+
+
+
+
+/* callback when config file is loaded */
+int tapeConfigLoadCbk (void *data)
+{
+  int rval = 1 ;
+  long iv ;
+  int bv ;
+  FILE *fp = (FILE *) data ;
+  char *dir ;
+
+  if (getString (topScope,"backlog-directory",&dir,NO_INHERIT))
+    {
+      if (tapeDirectory != NULL && strcmp (tapeDirectory,dir) != 0)
+        {
+          syslog (LOG_ERR,NO_CHANGE_BACKLOG) ;
+          dir = strdup (tapeDirectory) ;
+        }
+
+      if (!isDirectory (dir) && isDirectory (TAPE_DIRECTORY))
+        {
+          logOrPrint (LOG_ERR,fp,BAD_TAPEDIR_CHANGE,dir,TAPE_DIRECTORY) ;
+          dir = strdup (TAPE_DIRECTORY) ;
+        }
+      else if (!isDirectory (dir))
+        logAndExit (1,NO_TAPE_DIR) ;
+    }
+  else if (!isDirectory (TAPE_DIRECTORY))
+    logAndExit (1,NO_TAPE_DIR) ;
+  else
+    dir = strdup (TAPE_DIRECTORY) ;
+  
+  if (tapeDirectory != NULL)
+    FREE (tapeDirectory) ;
+  tapeDirectory = dir ;
+
+
+  
+  if (getInteger (topScope,"backlog-highwater",&iv,NO_INHERIT))
+    {
+      if (iv < 0)
+        {
+          rval = 0 ;
+          logOrPrint (LOG_ERR,fp,BLOGHIGH_NEG,iv,(long)TAPE_HIGHWATER);
+          iv = TAPE_HIGHWATER ;
+        }
+    }
+  else
+    iv = TAPE_HIGHWATER ;
+  tapeHighwater = (u_int) iv ;
+
+
+  
+  if (getInteger (topScope,"backlog-rotate-period",&iv,NO_INHERIT))
+    {
+      if (iv < 0)
+        {
+          rval = 0 ;
+          logOrPrint (LOG_ERR,fp,ROTATEPER_NEG,iv,(long)TAPE_ROTATE_PERIOD);
+          iv = TAPE_ROTATE_PERIOD ;
+        }
+    }
+  else
+    iv = TAPE_ROTATE_PERIOD ;
+  rotatePeriod = (u_int) iv ;
+
+
+  if (getInteger (topScope,"backlog-ckpt-period",&iv,NO_INHERIT))
+    {
+      if (iv < 0)
+        {
+          rval = 0 ;
+          logOrPrint (LOG_ERR,fp,CKPTPER_NEG,iv,
+                      (long)TAPE_CHECKPOINT_PERIOD);
+          iv = TAPE_CHECKPOINT_PERIOD ;
+        }
+    }
+  else
+    iv = TAPE_CHECKPOINT_PERIOD ;
+  tapeCkPtPeriod = (u_int) iv ;
+
+
+  if (getInteger (topScope,"backlog-newfile-period",&iv,NO_INHERIT))
+    {
+      if (iv < 0)
+        {
+          rval = 0 ;
+          logOrPrint (LOG_ERR,fp,CKNEWFILEPER_NEG,iv,
+                      (long)TAPE_NEWFILE_PERIOD);
+          iv = TAPE_NEWFILE_PERIOD ;
+        }
+    }
+  else
+    iv = TAPE_NEWFILE_PERIOD ;
+  tapeCkNewFilePeriod = (u_int) iv ;
+
+
+  if (getBool (topScope,"debug-shrinking",&bv,NO_INHERIT))
+    debugShrinking = (bv ? true : false) ;
+  
+  return rval ;
+}
+
+
 /* Create a new Tape object. There are three potential files involved in
-   I/O. 'peerName' is what the admin may have dropped in by
+   I/O. 'peerName' is what  the admin may have dropped in by
    hand. 'peerName.input' is the file that was being used as input the last
    time things were run. 'peerName.output' is the file that was being used
    as output. The file 'peerName' is appended to 'peerName.input' (or
@@ -212,21 +344,6 @@ Tape newTape (const char *peerName, bool dontRotate)
   ASSERT (nt->outputFilename != NULL) ;
   sprintf (nt->outputFilename,"%s/%s%s",tapeDirectory,peerName,OUTPUT_TAIL) ;
 
-  nt->inFp = NULL ;
-  nt->outFp = NULL ;
-  
-  nt->head = NULL ;
-  nt->tail = NULL ;
-
-  nt->scribbled = false ;
-  nt->tellpos = 0 ;
-  nt->qLength = 0 ;
-
-  nt->outputHasStuff = false ;
-  nt->outputFileExists = false ; /* it won't */
-
-  nt->changed = false ;
-  
   if ( !lockFile (nt->lockFilename) )
     {
       syslog (LOG_ERR,NO_LOCK_TAPE,nt->lockFilename) ;
@@ -239,13 +356,9 @@ Tape newTape (const char *peerName, bool dontRotate)
 
       return NULL ;
     }
-  
+
   nt->noRotate = false ;        /* for first time prepare */
-  nt->outputSize = 0 ;
-  nt->outputLimit = defaultSizeLimit ;
-
-  prepareFiles (nt) ;
-
+  initTape (nt) ;
   nt->noRotate = dontRotate ;
 
   addTapeGlobally (nt) ;
@@ -253,9 +366,100 @@ Tape newTape (const char *peerName, bool dontRotate)
   if (checkPtId == 0 && tapeCkPtPeriod > 0)     /* only done once. */
     checkPtId = prepareSleep (tapeCheckpointCallback,tapeCkPtPeriod,NULL);
 
+  if (ckNewFileId == 0 && tapeCkNewFilePeriod > 0)
+    ckNewFileId = prepareSleep (tapeCkNewFileCbk,tapeCkNewFilePeriod,NULL) ;
+
   return nt ;
 }
 
+static void initTape (Tape nt)
+{
+  value *peerVal = findPeer (nt->peerName) ;
+
+  nt->inFp = NULL ;
+  nt->outFp = NULL ;
+
+  nt->lastRotated = 0 ;
+  nt->checkNew = false ;
+  
+#if 0
+  nt->head = NULL ;
+  nt->tail = NULL ;
+  nt->qLength = 0 ;
+#endif
+
+  nt->scribbled = false ;
+  nt->tellpos = 0 ;
+
+  nt->changed = false ;
+  
+  nt->outputSize = 0 ;
+  nt->lossage = 0 ;
+
+  nt->backlogFactor = 0.0 ;
+  nt->outputLowLimit = 0 ;
+  nt->outputHighLimit = 0 ;
+
+  if (!talkToSelf)
+    {
+      if (getInteger (peerVal->v.scope_val,"backlog-limit",
+                      &nt->outputLowLimit,INHERIT))
+        {
+          if (!getReal (peerVal->v.scope_val,"backlog-factor",
+                        &nt->backlogFactor,INHERIT))
+            {
+              if (!getInteger (peerVal->v.scope_val,"backlog-limit-highwater",
+                               &nt->outputHighLimit,INHERIT))
+                {
+                  syslog (LOG_WARNING,NO_FACTOR,nt->peerName) ;
+                  nt->outputLowLimit = 0 ;
+                  nt->outputHighLimit = 0 ;
+                  nt->backlogFactor = 0.0 ;
+                }
+            }
+          else
+            nt->outputHighLimit = (long)(nt->outputLowLimit * nt->backlogFactor);
+        }
+      else
+        syslog (LOG_ERR,NODEFN,"backlog-limit") ;
+    }
+  
+  dprintf (1,"%s tape backlog limit: [%ld %ld]\n",nt->peerName,
+           nt->outputLowLimit,
+           nt->outputHighLimit) ;
+  
+  prepareFiles (nt) ;
+}
+
+
+void gFlushTapes (void)
+{
+  u_int i ;
+
+  syslog (LOG_NOTICE,FLUSHING_TAPES) ;
+  for (i = 0 ; i < activeTapeIdx ; i++)
+    tapeFlush (activeTapes [i]) ;
+}
+
+
+
+/* close the input and output tapes and reinitialize everything in the
+  tape. */
+void tapeFlush (Tape tape)
+{
+  if (tape->inFp != NULL)
+    {
+      checkpointTape (tape) ;
+      fclose (tape->inFp) ;
+    }
+
+  if (tape->outFp != NULL)
+    fclose (tape->outFp) ;
+
+  initTape (tape) ;
+}
+
+
 
 void gPrintTapeInfo (FILE *fp, u_int indentAmt)
 {
@@ -274,11 +478,38 @@ void gPrintTapeInfo (FILE *fp, u_int indentAmt)
   fprintf (fp,"%s}\n",indent) ;
 }
 
+
+void tapeLogGlobalStatus (FILE *fp)
+{
+  fprintf (fp,"Backlog file global values\n") ;
+  fprintf (fp,"        directory: %s\n",tapeDirectory) ;
+  fprintf (fp,"    rotate period: %-3ld seconds\n",(long) rotatePeriod) ;
+  fprintf (fp,"checkpoint period: %-3ld seconds\n",(long) tapeCkPtPeriod) ;
+  fprintf (fp,"   newfile period: %-3ld seconds\n",(long) tapeCkNewFilePeriod);
+  fprintf (fp,"\n") ;
+}
+
+
+void tapeLogStatus (Tape tape, FILE *fp)
+{
+  fprintf (fp,"                 backlog low limit: %ld\n",
+           tape->outputLowLimit) ;
+  fprintf (fp,"               backlog upper limit: %ld",
+           tape->outputHighLimit) ;
+  if (tape->backlogFactor > 0.0)
+    fprintf (fp," (factor %1.2f)",tape->backlogFactor) ;
+  fputc ('\n',fp) ;
+  fprintf (fp,"                 backlog shrinkage: %ld bytes (from current file)\n",
+           tape->lossage) ;
+}
+
 void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
 {
   char indent [INDENT_BUFFER_SIZE] ;
   u_int i ;
+#if 0
   QueueElem qe ;
+#endif
   
   for (i = 0 ; i < MIN(INDENT_BUFFER_SIZE - 1,indentAmt) ; i++)
     indent [i] = ' ' ;
@@ -299,9 +530,12 @@ void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
   fprintf (fp,"%s    peerName : %s\n",indent,tape->peerName) ;
   fprintf (fp,"%s    input-FILE : %p\n",indent, tape->inFp) ;
   fprintf (fp,"%s    output-FILE : %p\n",indent, tape->outFp) ;
+  fprintf (fp,"%s    output-limit : %ld\n",indent, tape->outputLowLimit) ;
 
+#if 0
   fprintf (fp,"%s    in-memory article queue (length  %d) {\n",indent,
            tape->qLength) ;
+
   for (qe = tape->head ; qe != NULL ; qe = qe->next)
     {
 #if 0
@@ -312,11 +546,8 @@ void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
     }
 
   fprintf (fp,"%s    }\n",indent) ;
+#endif
 
-  fprintf (fp,"%s    output-has-stuff : %s\n",indent,
-           boolToString (tape->outputHasStuff)) ;
-  fprintf (fp,"%s    output-file-exists : %s\n",indent,
-           boolToString (tape->outputFileExists)) ;
   fprintf (fp,"%s    tell-position : %ld\n",indent,(long) tape->tellpos) ;
   fprintf (fp,"%s    input-FILE-changed : %s\n",indent,
            boolToString (tape->changed)) ;
@@ -332,17 +563,32 @@ void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
 /* delete the tape. Spools the in-memory articles to disk. */
 void delTape (Tape tape)
 {
+  struct stat st ;
+  
   if (tape == NULL)
     return ;
 
-  /* flush out articles not yet gone to disk. */
-  if (tape->head != NULL)
-    {
-      tapeOpen (tape) ;         /* will flush articles to disk */
-      tapeClose (tape) ;
-    }
+#if 1
+  
+  if (tape->outFp != NULL && fclose (tape->outFp) != 0)
+    syslog (LOG_ERR,FCLOSE_FAILED, tape->outputFilename) ;
 
-  if (tape->inFp)
+  if (stat(tape->outputFilename, &st) == 0 && st.st_size == 0)
+    {
+      dprintf (1,"removing empty output tape: %s\n",tape->outputFilename) ;
+      (void) unlink (tape->outputFilename) ;
+    }
+  
+  tape->outFp = NULL ;
+  tape->outputSize = 0 ;
+
+#else
+  
+  tapeClose (tape) ;
+
+#endif
+
+  if (tape->inFp != NULL)
     {
       checkpointTape (tape) ;
       (void) fclose (tape->inFp) ;
@@ -362,40 +608,58 @@ void delTape (Tape tape)
 }
 
 
-/* Request the tape to hang onto the article information. If the output
-   side of the tape has been opened explicitly, then the article infor is
-   dumped straight to disk. Otherwise it's stored on a small queue. When
-   the queue gets too big it will be flushed to disk. */
 void tapeTakeArticle (Tape tape, Article article)
 {
+#if 0
   QueueElem elem ;
+#endif
+  int amt ;
 
   ASSERT (tape != NULL) ;
+
+  amt = fprintf (tape->outFp,"%s %s\n",artFileName (article),
+                 artMsgId (article)) ;
+
+  /* I'd rather know where I am each time, and I don't trust all
+    fprintf's to give me character counts. */
+#if defined (TRUST_FPRINTF)
+
+  tape->outputSize += amt ;
+
+#else
   
-  tape->outputHasStuff = true ;
+  tape->outputSize = ftell (tape->outFp) ;
+
+#endif
   
-  if (tape->outFp != NULL)      /* output size opened explicitly */
+  delArticle (article) ;
+
+  if (debugShrinking)
     {
-      tape->outputFileExists = true ;
-      
-      fprintf (tape->outFp,"%s %s\n",artFileName (article),
-               artMsgId (article)) ;
+      struct stat sb ;
 
-      /* I'd rather know where I am each time, and I don't trust all
-         fprintf's to give me character counts. */
-      tape->outputSize = ftell (tape->outFp) ;
-      
       fflush (tape->outFp) ;
-      delArticle (article) ;
-
-      if (tape->outputLimit > 0 &&
-          tape->outputSize >= (tape->outputLimit * LIMIT_FUDGE))
-        {
-          (void) shrinkfile (tape->outFp,
-                             tape->outputLimit,tape->outputFilename,"a+") ;
-          tape->outputSize = ftell (tape->outFp) ;
-        }
+      
+      if (fstat (fileno (tape->outFp),&sb) != 0)
+        syslog (LOG_ERR,FSTAT_FAILURE,tape->outputFilename) ;
+      else if (sb.st_size != tape->outputSize) 
+        syslog (LOG_ERR,"fstat and ftell do not agree: %ld %ld for %s\n",
+                sb.st_size,tape->outputSize) ;
     }
+  
+  if (tape->outputHighLimit > 0 && tape->outputSize >= tape->outputHighLimit)
+    {
+      long oldSize = tape->outputSize ;
+      (void) shrinkfile (tape->outFp,
+                         tape->outputLowLimit,tape->outputFilename,"a+") ;
+      tape->outputSize = ftell (tape->outFp) ;
+      tape->lossage += oldSize - tape->outputSize ;
+    }
+
+#if 0
+
+  XXXX
+    
   else                          /* stick on queue */
     {
       elem = ALLOC (struct q_e_s, 1) ;
@@ -414,10 +678,12 @@ void tapeTakeArticle (Tape tape, Article article)
         {
           dprintf (2,"Gone over tape HIGHWATER (%d). Flushing to disk %s\n",
                    TAPE_HIGHWATER, tape->outputFilename) ;
-          tapeOpen (tape) ;
-          tapeClose (tape) ;
+          flushTape (tape) ;
         }
     }
+
+#endif
+  
 }
 
 
@@ -430,10 +696,11 @@ Article getArticle (Tape tape)
   char *msgid, *filename ;
   Article art = NULL ;
   int c ;
+  time_t now = theTime() ;
 
   ASSERT (tape != NULL) ;
-  
-  if (tape->inFp == NULL)
+
+  if (tape->inFp == NULL && (now - tape->lastRotated) > rotatePeriod)
     prepareFiles (tape) ;       /* will flush queue too. */
 
   while (tape->inFp != NULL && art == NULL)
@@ -452,11 +719,13 @@ Article getArticle (Tape tape)
 
           dprintf (1,"No more articles on tape %s\n",tape->inputFilename) ;
 
-          (void) unlink (tape->inputFilename) ;
-
           tape->inFp = NULL ;
           tape->scribbled = false ;
-          prepareFiles (tape) ; /* rotate files to try next. */
+
+          (void) unlink (tape->inputFilename) ;
+
+          if ((now - tape->lastRotated) > rotatePeriod)
+            prepareFiles (tape) ; /* rotate files to try next. */
         }
       else
         {
@@ -501,30 +770,33 @@ Article getArticle (Tape tape)
           /* art may be NULL here if the file is no longer valid. */
         }
     }
-
   
+#if 0
   /* now we either have an article or there is no more on disk */
-  
-  if (tape->inFp != NULL && ((c = fgetc (tape->inFp)) != EOF))
-    ungetc (c,tape->inFp) ;
-  else if (tape->inFp != NULL)
+  if (art == NULL)
     {
-      /* last article read was the end of the tape. */
-      if (fclose (tape->inFp) != 0)
-        syslog (LOG_ERR,FCLOSE_FAILED,tape->inputFilename) ;
-      
-      tape->inFp = NULL ;
-      tape->scribbled = false ;
-
-      /* toss out the old input file and prepare the new one */
-      (void) unlink (tape->inputFilename) ;
-
-      prepareFiles (tape) ;
+      if (tape->inFp != NULL && ((c = fgetc (tape->inFp)) != EOF))
+        ungetc (c,tape->inFp) ; /* shouldn't happen */
+      else if (tape->inFp != NULL)
+        {
+          /* last article read was the end of the tape. */
+          if (fclose (tape->inFp) != 0)
+            syslog (LOG_ERR,FCLOSE_FAILED,tape->inputFilename) ;
+          
+          tape->inFp = NULL ;
+          tape->scribbled = false ;
+          
+          /* toss out the old input file and prepare the new one */
+          (void) unlink (tape->inputFilename) ;
+          
+          if (now - tape->lastRotated > rotatePeriod)
+            prepareFiles (tape) ;
+        }
     }
+#endif
   
   if (art == NULL)
-    dprintf (2,"%s All out of articles in the backlog\n",
-             tape->peerName) ;
+    dprintf (2,"%s All out of articles in the backlog\n", tape->peerName) ;
   else
     dprintf (2,"%s Peeled article %s from backlog\n",tape->peerName,
              artMsgId (art)) ;
@@ -533,63 +805,36 @@ Article getArticle (Tape tape)
 }
 
 
+#if 0
 /* (re)open a tape for output. */
 bool tapeOpen (Tape tape)
 {
-  QueueElem elem ;
-
   ASSERT (tape != NULL) ;
 
-  if (tape->outFp == NULL)
+  if (tape->outFp != NULL)
+    fclose (tape->outFp) ;
+  
+  if ((tape->outFp = fopen (tape->outputFilename,"a+")) == NULL)
     {
-      if ((tape->outFp = fopen (tape->outputFilename,"a+")) == NULL)
-        {
-          syslog (LOG_ERR,TAPE_OPEN_FAILED, "a+", tape->outputFilename) ;
-          return false ;
-        }
-
-      tape->outputSize = ftell (tape->outFp) ;
+      syslog (LOG_ERR,TAPE_OPEN_FAILED, "a+", tape->outputFilename) ;
+      return false ;
     }
-
-  /* flush out queue to disk. */
-  elem = tape->head ;
-  while (elem != NULL)
-    {
-      tape->head = tape->head->next ;
-      fprintf (tape->outFp,"%s %s\n", artFileName (elem->article),
-               artMsgId (elem->article)) ;
-
-      tape->outputFileExists = true ;
-      delArticle (elem->article) ;
-      
-      FREE (elem) ;
-      elem = tape->head ;
-    }
-
-  tape->outputSize = ftell (tape->outFp) ;
-  tape->tail = NULL ;
-  tape->qLength = 0;
-      
-  fflush (tape->outFp) ;
-
-  if (tape->outputLimit > 0 &&
-      tape->outputSize >= (tape->outputLimit * LIMIT_FUDGE))
-    {
-      (void) shrinkfile (tape->outFp,
-                         tape->outputLimit,tape->outputFilename,"a+") ;
-      tape->outputSize = ftell (tape->outFp) ;
-    }
-
+  
+  flushTape (tape) ;
+  
   return true ;
 }
+#endif
 
-
+#if 0
 /* close the output side of the tape. */
 void tapeClose (Tape tape)
 {
   struct stat st;
   
   ASSERT (tape != NULL) ;
+
+  flushTape (tape) ;
   
   if (tape->outFp != NULL && fclose (tape->outFp) != 0)
     syslog (LOG_ERR,FCLOSE_FAILED, tape->outputFilename) ;
@@ -603,9 +848,10 @@ void tapeClose (Tape tape)
   tape->outFp = NULL ;
   tape->outputSize = 0 ;
 }
+#endif
 
 
-
+#if 0
 /* return true if there are articles on the tape somewhere avialable for
    processing. In the NOROTATE case we only look at the input side. */
 bool tapeHasArticles (Tape tape)
@@ -617,11 +863,12 @@ bool tapeHasArticles (Tape tape)
   if (tape->inFp != NULL)
     rval = true ;
   
-  if ((tape->outputFileExists || tape->head != NULL) && !tape->noRotate)
+  if ((tape->outputSize > 0 || tape->head != NULL) && !tape->noRotate)
     rval = true ;
   
   return rval ;
 }
+#endif
 
 
 /****************************************************/
@@ -629,12 +876,14 @@ bool tapeHasArticles (Tape tape)
 /****************************************************/
 
 
+#if 0
 /* set the number of seconds between checkpoints of the input side of a
    tape. */
 void tapeSetCheckpointPeriod (u_int ckPtPeriod)
 {
   tapeCkPtPeriod = ckPtPeriod ;
 }
+#endif
 
 
 /* Cause all the Tapes to checkpoint themselves. */
@@ -647,35 +896,60 @@ void checkPointTapes (void)
 }
 
 
-/* Set the pathname of the directory for storing tapes in. can only set
-   it once, and before any tapes are created.  */
+/* make all the tapes set their checkNew flag. */
+static void tapesSetCheckNew (void) 
+{
+  u_int i ;
+
+  for (i = 0 ; i < activeTapeIdx ; i++)
+    activeTapes[i]->checkNew = true ;
+}
+
+
+#if 0
+/* Set the pathname of the directory for storing tapes in. */
 void setTapeDirectory (const char *newDir)
 {
-  ASSERT (tapeDirectory == NULL) ;
-
+  /* the activeTape variable gets set when the first Tape object
+     is created */
+  if (activeTapes != NULL)
+    {
+      syslog (LOG_CRIT,"Resetting backlog directory") ;
+      abort() ;
+    }
+  
+  if (tapeDirectory != NULL)
+    freeCharP (tapeDirectory) ;
+  
   tapeDirectory = strdup (newDir) ;
 
   addPointerFreedOnExit (tapeDirectory) ;
 }
-
+#endif
 
 /* Get the pathname of the directory tapes are stored in. */
-const char *getTapeDirectory ()
+const char *getTapeDirectory (void)
 {
+  ASSERT (tapeDirectory != NULL) ;
+
+#if 0
   if (tapeDirectory == NULL)
     {
       tapeDirectory = strdup (TAPE_DIRECTORY) ;
       addPointerFreedOnExit (tapeDirectory) ;
     }
-
+#endif
+  
   return tapeDirectory ;
 }
 
 
+#if 0
 void setOutputSizeLimit (long val)
 {
   defaultSizeLimit = val ;
 }
+#endif
 
 
 
@@ -802,21 +1076,22 @@ static void checkpointTape (Tape tape)
 
 
 
-/* Prepare the tape file(s) for input. For a given Tape there are
- * three possible input files: PEER.input PEER and
+/* Prepare the tape file(s) for input and output */
+
+/* For a given Tape there are
+ * three possible files: PEER.input PEER and
  * PEER.output. PEER.input and PEER.output are private to
  * innfeed. PEER is where a sysadmin can drop a file that (s)he
  * wants to inject into the process. If the first line of the input file
  * contains only an integer (possibly surrounded by spaces), then this is
- * taken to be the position to seek to
+ * taken to be the position to seek to before reading.
  *
- * prepareFiles will them in a manner much like the following shell
+ * prepareFiles will process them in a manner much like the following shell
  * commands:
  *
  * if [ ! -f PEER.input ]; then
- * 	if [ -f PEER ]; then mv PEER PEER.input; fi
+ * 	if [ -f PEER ]; then mv PEER PEER.input
  * 	elif [ -f PEER.output ]; then mv PEER.output PEER; fi
- * 	rm PEER.checkpoint
  * fi
  *
  * At this point PEER.input is opened for reading if it exists.
@@ -830,18 +1105,35 @@ static void prepareFiles (Tape tape)
   bool outExists ;
   bool newExists ;
 
+#if 0
   /* flush any in memory articles to disk */
-  if (tape->head != NULL)
-    {
-      tapeOpen (tape) ;
-      tapeClose (tape) ;
-    }
+  if (tape->head != NULL && tape->outFp != NULL)
+    flushTape(tape) ;
+#endif
 
   tape->tellpos = 0 ;
+
+  /*  First time through, or something external has set checkNew */
+  if (tape->lastRotated == 0 || tape->checkNew)
+    {
+      newExists = fileExistsP (tape->handFilename) ;
+      if (newExists)
+        syslog (LOG_NOTICE,NEW_HAND_FILE,tape->peerName) ;
+    }
+  else
+    newExists = false ;
+  
+  if (tape->lastRotated == 0)    /* first time here */
+    {
+      inpExists = fileExistsP (tape->inputFilename) ;
+      outExists = fileExistsP (tape->outputFilename) ;
+    }
+  else
+    {
+      inpExists = (tape->inFp != NULL) ; /* can this ever be true?? */
+      outExists = (tape->outFp != NULL && tape->outputSize > 0) ;
+    }
   
-  inpExists = fileExistsP (tape->inputFilename) ;
-  outExists = fileExistsP (tape->outputFilename) ;
-  newExists = fileExistsP (tape->handFilename) ;
 
   /* move the hand-dropped file to the input file if needed. */
   if (newExists && !inpExists)
@@ -849,20 +1141,29 @@ static void prepareFiles (Tape tape)
       if (rename (tape->handFilename,tape->inputFilename) != 0)
         syslog (LOG_ERR,RENAME_FAILED,tape->handFilename, tape->inputFilename);
       else
-        inpExists = true ;
+        {
+          syslog (LOG_NOTICE,ROTATING_HAND_DROPPED,tape->peerName) ;
+          inpExists = true ;
+        }
     }
   
   /* now move the output file to the input file, if needed and only if in
      not in NOROTATE mode. */
   if (outExists && !inpExists && !tape->noRotate)
     {
+      if (tape->outFp != NULL)
+        {
+          fclose (tape->outFp) ;
+          tape->outFp = NULL ;
+        }
+      
       if (rename (tape->outputFilename,tape->inputFilename) != 0)
         syslog (LOG_ERR,RENAME_FAILED,
                 tape->outputFilename,tape->inputFilename);
       else
         inpExists = true ;
-      
-      tape->outputFileExists = false ;
+
+      outExists = false ;
     }
 
   /* now open up the input file and seek to the proper position. */
@@ -879,9 +1180,17 @@ static void prepareFiles (Tape tape)
 
           if (fgets (buffer,sizeof (buffer) - 1, tape->inFp) == NULL)
             {
-              syslog (LOG_ERR,FGETS_FAILED,tape->inputFilename) ;
+              if (feof (tape->inFp))
+                {
+                  dprintf (1,"Empty input file: %s",tape->inputFilename) ;
+                  unlink (tape->inputFilename) ;
+                }
+              else
+                syslog (LOG_ERR,FGETS_FAILED,tape->inputFilename) ;
+              
               fclose (tape->inFp) ;
               tape->inFp = NULL ;
+              tape->scribbled = false ;
             }
           else
             {
@@ -899,47 +1208,84 @@ static void prepareFiles (Tape tape)
                       tape->tellpos = newPos ;
                     }
                 }
-            }
-        }
-      
-      if ((flength = fileLength (fileno (tape->inFp))) < tape->tellpos)
-        {
-          syslog (LOG_ERR,FILE_SHORT,tape->inputFilename,
-                  flength, tape->tellpos);
-          tape->tellpos = 0 ;
-        }
-      else if (tape->tellpos == 0)
-        rewind (tape->inFp) ;
-      else if (fseek (tape->inFp,tape->tellpos - 1,SEEK_SET) != 0)
-        syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos);
-      else if ((c = fgetc (tape->inFp)) != '\n')
-        {
-          while (c != EOF && c != '\n')
-            c = fgetc (tape->inFp) ;
 
-          if (c == EOF)
-            {
-              fclose (tape->inFp) ;
-              unlink (tape->inputFilename) ;
-              tape->inFp = NULL ;
-              tape->scribbled = false ;
-              prepareFiles (tape) ;
-            }
-          else
-            {
-              long oldPos = tape->tellpos ;
-
-              tape->changed = true ;
-              checkpointTape (tape) ;
-              
-              syslog (LOG_ERR,CKPT_BNDRY,tape->inputFilename,
-                      tape->tellpos, oldPos) ;
+              if ((flength = fileLength (fileno (tape->inFp))) < tape->tellpos)
+                {
+                  syslog (LOG_ERR,FILE_SHORT,tape->inputFilename,
+                          flength, tape->tellpos);
+                  tape->tellpos = 0 ;
+                }
+              else if (tape->tellpos == 0)
+                rewind (tape->inFp) ;
+              else if (fseek (tape->inFp,tape->tellpos - 1,SEEK_SET) != 0)
+                syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos);
+              else if ((c = fgetc (tape->inFp)) != '\n')
+                {
+                  while (c != EOF && c != '\n')
+                    c = fgetc (tape->inFp) ;
+                  
+                  if (c == EOF)
+                    {
+                      fclose (tape->inFp) ;
+                      unlink (tape->inputFilename) ;
+                      tape->inFp = NULL ;
+                      tape->scribbled = false ;
+                      prepareFiles (tape) ;
+                    }
+                  else
+                    {
+                      long oldPos = tape->tellpos ;
+                      
+                      tape->changed = true ;
+                      checkpointTape (tape) ;
+                      
+                      syslog (LOG_ERR,CKPT_BNDRY,tape->inputFilename,
+                              tape->tellpos, oldPos) ;
+                    }
+                }
             }
         }
     }
+  
+  tape->lastRotated = theTime() ;
+  tape->checkNew = false ;
+
+  /* now open up the output file. */
+#if 0
+  
+  tapeOpen (tape) ;
+
+#else
+
+  if (tape->outFp == NULL)
+    {
+      if ((tape->outFp = fopen (tape->outputFilename,"a+")) == NULL)
+        {
+          syslog (LOG_ERR,TAPE_OPEN_FAILED, "a+", tape->outputFilename) ;
+          return ;
+        }
+      fseek (tape->outFp,0,SEEK_END) ;
+      tape->outputSize = ftell (tape->outFp) ;
+      tape->lossage = 0 ;
+    }
+  
+#endif
+  
 }
 
 
+static void tapeCkNewFileCbk (TimeoutId id, void *d)
+{
+  (void) d ;
+  
+  ASSERT (id == ckNewFileId) ;
+  ASSERT (tapeCkNewFilePeriod > 0) ;
+
+  tapesSetCheckNew () ;
+
+  ckNewFileId = prepareSleep (tapeCkNewFileCbk,tapeCkNewFilePeriod,NULL) ;
+}
+
 
 /* The timer callback function that will checkpoint all the active tapes. */
 static void tapeCheckpointCallback (TimeoutId id, void *d)
@@ -958,3 +1304,54 @@ static void tapeCheckpointCallback (TimeoutId id, void *d)
 
 
 
+#if 0
+static void flushTape (Tape tape)
+{
+  QueueElem elem ;
+
+  /* flush out queue to disk. */
+  elem = tape->head ;
+  while (elem != NULL)
+    {
+      tape->head = tape->head->next ;
+      fprintf (tape->outFp,"%s %s\n", artFileName (elem->article),
+               artMsgId (elem->article)) ;
+      
+      delArticle (elem->article) ;
+      
+      FREE (elem) ;
+      elem = tape->head ;
+    }
+  tape->tail = NULL ;
+  tape->qLength = 0;
+      
+
+  /* I'd rather know where I am each time, and I don't trust all
+     fprintf's to give me character counts. */
+  tape->outputSize = ftell (tape->outFp) ;
+  if (debugShrinking)
+    {
+      struct stat buf ;
+      static bool logged = false ;
+      
+      fflush (tape->outFp) ;
+      if (fstat (fileno (tape->outFp),&buf) != 0 && !logged)
+        {
+          syslog (LOG_ERR,FSTAT_FAILURE,tape->outputFilename) ;
+          logged = true ;
+        }
+      else if (buf.st_size != tape->outputSize)
+        {
+          syslog (LOG_ERR,FSTAT_NE_FTELL,tape->outputFilename) ;
+          logged = true ;
+        }
+    }
+  
+  if (tape->outputHighLimit > 0 && tape->outputSize > tape->outputHighLimit)
+    {
+      (void) shrinkfile (tape->outFp,
+                         tape->outputLowLimit,tape->outputFilename,"a+") ;
+      tape->outputSize = ftell (tape->outFp) ;
+    }
+}
+#endif
